
// include themes: these set variables for common values
@import "bootswatch_litera_theme.scss";
@import "openqa_theme.scss";

// include Roboto font
@import url($web-font-path);

// include Bootstrap 4
@import "../cache/raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/bootstrap.scss";

// include custom css for various aspects of the pages
@import "overall.scss";
@import "navigation.scss";
@import "forms.scss";
@import "tables.scss";
@import "dashboard.scss";
@import "test-details.scss";
@import "result_preview.scss";
@import "admin-pages.scss";
@import "comments.scss";
@import "overview.scss";
@import "parent_group_overview.scss";
@import "shepherd.scss";
