.table.overview {
    table-layout: fixed;
    tr.parallel-child td.name {
        padding-left: 1em;
        &:first-child:before {
            content: "•";
        }
    }
    a.fa, span > a {
        text-decoration: none;
    }
    .toggle-parallel-children {
        border-radius: 5rem;
    }
}
.collapse-all-button, .expand-all-button {
    float: right;
    text-decoration: none !important;
    border-radius: 5rem;
    margin-left: 0.5rem;
}

#summary .badge {
    font-size: 90%;
    padding: 4px 6px 4px;
    border-radius: 0.5rem;
    color: #fff;
    background-color: #bbb;
}
#summary .time-params {
    font-weight: lighter;
}
@include media-breakpoint-up(xl) {
    #summary .time-params {
        float: right;
    }
}
.darkmode {
    #summary .time-params {
        font-weight: bold;
    }
}

/* dotted underline looks strange with default font size */
.h4 {
    .smaller-font { font-size: 90%; }
}

#filter-panel, #developer-panel, #live-log-panel, #live-terminal-panel {
    .card-header {
        cursor: pointer;

        span {
            color: #555;
            font-weight: 500;
            float: right;
        }
        .row span {
            text-align: right;
            width: 100%;
            display: inline-block;
            float: none;
        }
    }
    .card-header-appendix, .card-header-appendix span {
        font-weight: normal;
        float: none;
    }
    .card-body, .developer-mode-element {
        display: none;
    }

    #filter-form {
        strong {
            display: block;
        }
        .form-group label {
            margin-right: 3px;
            font-weight: normal;
        }
    }
}
.darkmode {
    #filter-panel, #developer-panel, #live-log-panel, #live-terminal-panel {
        .card-header {
            span {
                color: rgb(214, 214, 214);
            }
        }
        .card-body, .developer-mode-element {
            color: $default-font-color-darktheme;
            background-color: $default-bg-color-darktheme;
        }
    }
}

/* workaround to avoid issue with Chosen select when placeholder is not shown by default */
.chosen-search-input.default {
    width: 100% !important;
}

#filter-modules label {
    margin-bottom: 0;
}
