.comment-body {
    .comment-editing-control {
        display: none;
    }

    textarea.comment-editing-control {
        outline: none;
        border: none;
        padding: 0px;
        margin: 0px;
        color: #000;
    }

    .comment-buttons {
        margin-top: -10px;
        margin-bottom: 15px;
    }

    .trigger-edit-button, .remove-edit-button {
        float: right;
        margin-top: -2px;
        margin-left: 4px;
        border-radius: 50rem;
        width: 2.2rem;
        height: 2.2rem;
        opacity: 0.6;

        &:hover {
            opacity: 1;
        }
    }

    .comment-anchor {
        color: #444;
        cursor: pointer;

        abbr {
            cursor: pointer;
        }

        &:hover {
            text-decoration: none;

            abbr {
                border-bottom: 1px solid #444;
            }
        }
    }

    .openqa-label {
        position: relative;
        background-color: #fdfd96;
        color: #666666;
        padding: .3em;
        padding-left: 1.25em;
        border-radius: .25rem;
        border: 1px solid #ecec85;
    }

    .openqa-label::before {
        display: inline-block;
        font-style: normal;
        font-variant: normal;
        text-rendering: auto;
        font-family: "ForkAwesome";
        font-weight: 900;
        font-size: .8em;
        content: "\f02e";
        position: absolute;
        left: 0;
        top: 50%;
        transform: translateY(-50%);
        width: 1.25em;
        text-align: center;
        padding-left: .25em;
    }

    .well {
        margin: 0px;
    }
}

.darkmode {
    .comment-body .comment-anchor {
        color: rgb(152, 152, 152);
    }
}

.comment-row {
    align-items: center;
    margin-bottom: 1.25rem;
    .col-sm-1 {
        text-align: right;
    }
}

.pagination {
    display: inline-block;

    li {
        color: $btn-default-color;
        float: left;
        text-decoration: none;
        transition: background-color .3s;
    }

    .active {
        font-weight: bold;
    }
}

.comment-toolbar {
    display: grid;
    padding: 0.5rem;
    align-items: center;
}

.darkmode {
    .input-group-text {
        background-color: $default-bg-color-darktheme;
    }
}
