! bootstrap.css
# download the fonts
< https://cdn.datatables.net/1.10.16/css/dataTables.bootstrap4.css
< https://cdn.jsdelivr.net/npm/fork-awesome@1.2.0/css/fork-awesome.min.css
< stylesheets/openqa.scss
< https://raw.githubusercontent.com/bootstrapthemesco/bootstrap-4-multi-dropdown-navbar/beta2.0/css/bootstrap-4-navbar.css
< https://cdnjs.cloudflare.com/ajax/libs/chosen/1.7.0/chosen.css
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/bootstrap.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_functions.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_forms.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_variables.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_mixins.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_breakpoints.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_deprecate.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_hover.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_image.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_badge.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_resize.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_screen-reader.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_size.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_reset-text.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_text-emphasis.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_text-hide.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_text-truncate.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_visibility.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_alert.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_buttons.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_caret.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_pagination.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_lists.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_list-group.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_nav-divider.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_forms.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_table-row.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_background-variant.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_border-radius.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_box-shadow.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_gradients.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_transition.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_clearfix.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_grid-framework.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_grid.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/mixins/_float.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_root.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_reboot.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_type.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_images.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_code.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_grid.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_tables.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_tables.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_toasts.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_buttons.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_transitions.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_dropdown.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_button-group.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_input-group.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_custom-forms.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_nav.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_navbar.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_card.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_breadcrumb.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_pagination.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_spinners.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_badge.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_jumbotron.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_alert.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_progress.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_media.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_list-group.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_close.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_modal.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_tooltip.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_popover.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_carousel.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_utilities.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_align.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_background.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_borders.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_clearfix.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_display.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_embed.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_flex.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_float.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_interactions.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_overflow.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_position.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_screenreaders.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_shadows.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_sizing.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_spacing.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_stretched-link.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_text.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/utilities/_visibility.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/vendor/_rfs.scss
<< https://raw.githubusercontent.com/twbs/bootstrap/v4.6.1/scss/_print.scss
< stylesheets/back_to_top.css

! dagre-d3.js
< https://d3js.org/d3.v4.js
< https://raw.githubusercontent.com/dagrejs/dagre-d3/v0.5.0/dist/dagre-d3.js

! dependency_graph.css
< stylesheets/dependency_graph.scss

! codemirror.css
< https://raw.githubusercontent.com/codemirror/CodeMirror/8097c7e75ce7ef0512debe9967d7568626106e53/lib/codemirror.css
< stylesheets/codemirror_customization.css

! codemirror.js
< https://raw.githubusercontent.com/codemirror/CodeMirror/8097c7e75ce7ef0512debe9967d7568626106e53/lib/codemirror.js
< https://raw.githubusercontent.com/codemirror/CodeMirror/8097c7e75ce7ef0512debe9967d7568626106e53/mode/perl/perl.js
< https://raw.githubusercontent.com/codemirror/CodeMirror/8097c7e75ce7ef0512debe9967d7568626106e53/mode/yaml/yaml.js
< https://raw.githubusercontent.com/codemirror/CodeMirror/8097c7e75ce7ef0512debe9967d7568626106e53/mode/diff/diff.js

! step_edit.js
< javascripts/needleeditor.js
< javascripts/shapes.js

! bootstrap.js
< https://code.jquery.com/jquery-3.6.0.js
< javascripts/openqa.js
< https://cdn.datatables.net/1.10.16/js/jquery.dataTables.js
< https://cdn.datatables.net/1.10.16/js/dataTables.bootstrap4.js
< http://timeago.yarp.com/jquery.timeago.js
< javascripts/index.js
< javascripts/admintable.js
< javascripts/admin_user.js
< javascripts/admin_api_keys.js
< javascripts/admin_needle.js
< javascripts/admin_worker.js
< javascripts/admin_groups.js
< javascripts/admin_assets.js
< javascripts/audit_log.js
< javascripts/back_to_top.js
< javascripts/tests.js
< javascripts/job_templates.js
< javascripts/overview.js
< javascripts/filter_form.js
< javascripts/comments.js
< javascripts/keyevent.js
< javascripts/feature.js
< javascripts/fullscreen.js
< javascripts/obs_rsync.js
< javascripts/parent_group_overview.js
< https://raw.githubusercontent.com/bootstrapthemesco/bootstrap-4-multi-dropdown-navbar/beta2.0/js/bootstrap-4-navbar.js
< https://cdnjs.cloudflare.com/ajax/libs/popper.js/1.16.1/umd/popper.js
< https://cdn.jsdelivr.net/npm/bootstrap@4.6.1/dist/js/bootstrap.js
< https://cdnjs.cloudflare.com/ajax/libs/chosen/1.7.0/chosen.jquery.js
< https://cdnjs.cloudflare.com/ajax/libs/jquery-ujs/1.2.1/rails.js
< javascripts/disable_animations.js [mode==test]
< https://shepherdjs.dev/dist/js/shepherd.js

! anser.js
< javascripts/anser-import.js
< https://raw.githubusercontent.com/IonicaBizau/anser/2.0.1/lib/index.js

! test_result.js
< javascripts/render.js
< javascripts/test_result.js
< javascripts/needlediff.js
< javascripts/running.js
< javascripts/disable_status_updates.js [mode==test]

! job_next_previous.js
< javascripts/job_next_previous.js

! video.css
< stylesheets/video.scss

! ws_console.css
< stylesheets/ws_console.scss

! ansi-colors.css
< stylesheets/ansi-colors.scss

! ws_console.js
< javascripts/ws_console.js

! logo-16.png
< images/logo-16.png

! logo-none-16.png
< images/logo-16.png

! logo-scheduled-16.png
< images/logo-scheduled-16.png

! logo-blocked-16.png
< images/logo-blocked-16.png

! logo-execution-16.png
< images/logo-execution-16.png

! logo-cancelled-16.png
< images/logo-cancelled-16.png

! logo-passed-16.png
< images/logo-passed-16.png

! logo-softfailed-16.png
< images/logo-softfailed-16.png

! logo-failed-16.png
< images/logo-failed-16.png

! logo-not_complete-16.png
< images/logo-not_complete-16.png

! logo-aborted-16.png
< images/logo-cancelled-16.png

! logo.svg
< images/logo.svg

! logo-none.svg
< images/logo.svg

! logo-scheduled.svg
< images/logo-scheduled.svg

! logo-blocked.svg
< images/logo-blocked.svg

! logo-execution.svg
< images/logo-execution.svg

! logo-cancelled.svg
< images/logo-cancelled.svg

! logo-passed.svg
< images/logo-passed.svg

! logo-softfailed.svg
< images/logo-softfailed.svg

! logo-failed.svg
< images/logo-failed.svg

! logo-not_complete.svg
< images/logo-not_complete.svg

! logo-aborted.svg
< images/logo-cancelled.svg

! terminal.svg
< images/terminal.svg

! suse.png
< images/suse.png

! audio.svg
< images/audio.svg
