body {
    font-family: $default-font;
    line-height: 1.428571429;
    -webkit-font-smoothing: antialiased;
}

// overall appearance (background color, font color, paragraphs, footer, etc.)
html {
    position: relative;
    min-height: 100%;
}
body {
    color: $default-font-color;
    background-color: $default-bg-color;
    letter-spacing: .1px;

    // always show a scrollbar not to flicker on tabs / long drop downs
    overflow-y: scroll;
}
.darkmode {
    color: $default-font-color-darktheme;
    background-color: $default-bg-color-darktheme;
}
@include media-breakpoint-up(md) {
    #content {
        padding-left: 40px;
        padding-right: 40px;
    }
}

// footer
.footer {
    position: absolute;
    bottom: 0;
    width: 100%;
    // set the fixed height of the footer here
    height: 30px;
    background-color: #f5f5f5;
}
.darkmode {
    .footer {
        background-color: $default-bg-color-darktheme;
    }
}
#footer-links {
    &.left {
        float: left;
    }

    &.right {
        float: right;
    }

    a {
        display: inline;
        margin-left: 15px;
        margin-right: 15px;
    }
}
body {
    // add some space between last elements and footer
    margin-bottom: 45px;
}

// paragraphs, links, misc
p {
    margin: 0 0 1em;
}
a {
    @include transition(all 0.2s);
}
.darkmode {
    a {
        color: $link-color-darktheme;
    }
}
.links {
    text-align: left;
    padding-top: 0;
}

// headings
h1, .h1, h2, .h2 {
    margin-top: 18px;
    margin-bottom: 9px;
}

h1, h2, h3, h4, h5, h6, .h1, .h2, .h3, .h4, .h5, .h6 {
    font-family: inherit;
    font-weight: 400;
    line-height: 1.1;
    color: #444;
}
.darkmode {
    h1, h2, h3, h4, h5, h6, .h1, .h2, .h3, .h4, .h5, .h6 {
        color: #bbb;
    }
}

// cards (former panels)
.card {
    margin-bottom: 15px;
}
.darkmode {
    .card {
        background-color: $default-bg-color-darktheme;
        border: $table-border-color-darktheme solid 0.5px;
    }
}
// colorful card headers (Bootstrap 4 only has colorful borders or fully colored cards)
.card.border-danger {
    border-color: #eed3d7;
    .card-header {
        color: #b94a48;
        background-color: #f2dede;
    }
}
.darkmode {
    .card.border-danger .card-header {
        color: #cf4644;
        background-color: #450202;
    }
}
.card.border-success {
    border-color: #d6e9c6;
    .card-header {
        color: #468847;
        background-color: #dff0d8;
    }
}
.darkmode {
    .card.border-success .card-header {
        color: #74f476;
        background-color: #144500;
    }
}
.card.border-warning {
    border-color: #fbeed5;
    .card-header {
        color: #c09853;
        background-color: #fcf8e3;
    }
}
.darkmode {
    .card.border-warning .card-header {
        color: #fdca71;
        background-color: #1d1800;
    }
}
.card.border-info {
    border-color: #bce8f1;
    .card-header {
        color: #3a87ad;
        background-color: #d9edf7;
    }
}
.darkmode {
    .card.border-info .card-header {
        color: #64cafd;
        background-color: #000000;
    }
}

// big welcome box ("jumbtron")
.jumbotron {
    padding-top: 30px;
    padding-bottom: 30px;
    margin-bottom: 30px;
    color: inherit;
    background-color: #f9f9f9;
}
.darkmode {
    .jumbotron {
        background-color: #111111;
    }
}
.jumbotron p {
    margin-bottom: 15px;
    font-size: 20px;
    font-weight: 200;
}

// flash messages
#flash-messages, #editor_warnings {
    box-sizing: border-box;
    width: 100%;
}

// tabs
.tab-content {
    padding-top: 15px;
}
.nav-tabs .nav-link {
    border: none;
    color: $default-font-color;

    &:hover, &:focus, &.active {
        border: none;
        -webkit-box-shadow: inset 0 -2px 0 #446E9B;
        box-shadow: inset 0 -2px 0 #446E9B;
        color: #446E9B;
    }
}
.darkmode {
    .nav-tabs .nav-link {
        color: $default-font-color-darktheme;

        &:hover, &:focus, &.active {
            -webkit-box-shadow: inset 0 -2px 0 #6caef5;
            box-shadow: inset 0 -2px 0 #6caef5;
            color: #6caef5;
            background-color: $default-bg-color-darktheme;
        }
    }
}

// popover
.help_popover  {
    margin: 0px 5px;
    &:focus, &:hover {
        outline: none;
        text-decoration: none;
    }
}
.popover code {
    word-break: break-all;
}
.popover .popover-header {
    word-break: break-all;
}

// tooltips
.tooltip .tooltip-inner {
    max-width: 400px;
    p {
        margin-bottom: 2px;
    }
    ul {
        padding-left: 1.2rem;
    }
    p, ul {
        text-align: left;
        word-break: break-word;
    }
}

// full screen mode
#content_fullscreen {
    min-width: 100%;
}

.darkmode {
    .table {
        color: $default-font-color-darktheme;
        background-color: $default-bg-color-darktheme;
    }
}

.darkmode {
    // All tests pagination
    .page-link,.page-item.disabled .page-link {
        background-color: $default-bg-color-darktheme;
        color: $default-font-color-darktheme;
        border:rgba(255, 255, 255, 0.3) solid 0.5px;
    }

    // All tests search field
    .chosen-container .chosen-drop, .chosen-container .chosen-results li, .chosen-container ul.chosen-choices {
        background-color: $default-bg-color-darktheme;
        background-image: none;
        color: $default-font-color-darktheme;
    }

     // All tests table rows
    .table td {
        border-top-color: rgba(255, 255, 255, 0.03);
    }

    .modal.show {
        background-color: lighten($default-bg-color-darktheme, 25%);
    }
    .modal-content {
        background-color: $default-bg-color-darktheme;
    }
    .modal-header, .modal-footer {
        border-top: none;
        border-bottom: none;
    }

    .container-fluid .btn {
        color: $default-font-color-darktheme;
    }
    .container-fluid .btn-default:hover, .container-fluid .btn-default:active:hover {
        background-color: #101010;
        color: #cbc5bf;
    }
    .container-fluid .btn-default:active {
        background-color: #101010;
        background-image: radial-gradient(circle, lighten(#101010, 25%) 10%, #363636 11%);
    }

    input, textarea {
        background-color: $default-bg-color-darktheme;
        color: $default-font-color-darktheme;
        border: $table-border-color-darktheme solid 0.5px;
    }
}
